==> klayout-0.10.tar.gz: klayout-0.10/src/dbCellGraph.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef HDR_dbCellGraph
#define HDR_dbCellGraph

#include <dbObject.h>
#include <dbCell.h>
#include <tlException.h>
#include <tlVector.h>

#include <list>
#include <vector>

namespace db
{

class MemStatistics;
template <class Coord> class generic_repository;
typedef generic_repository<db::Coord> GenericRepository;

/**
 *  @brief A cell graph class
 *
 *  The cell graph is a container for a cell graph.
 *  The cell graph is constructed by creating cells
 *  and adding child instances to it.
 */

class CellGraph
  : public db::Object 
{
public:
  typedef db::Box box_type;
  typedef cell_inst<CellGraph> cell_inst_type;
  typedef cell<CellGraph> cell_type;
  typedef std::list<cell_type> cell_list;
  typedef cell_list::iterator iterator;
  typedef cell_list::const_iterator const_iterator;
  typedef unsigned int cell_index_type;
  typedef tl::vector<cell_index_type> cell_index_vector;
  typedef cell_index_vector::reverse_iterator bottom_up_iterator;
  typedef cell_index_vector::const_reverse_iterator bottom_up_const_iterator;
  typedef cell_index_vector::iterator top_down_iterator;
  typedef cell_index_vector::const_iterator top_down_const_iterator;
  typedef tl::vector<cell_type *> cell_ptr_vector;

  /**
   *  @brief Default ctor: create an empty graph
   */
  CellGraph (db::Manager *manager = 0);

  /**
   *  @brief Copy ctor
   */
  CellGraph (const CellGraph &d);

  /**
   *  @brief Assignment operator
   */
  CellGraph &operator= (const CellGraph &d);

  /**
   *  @brief Translation operator
   */
  void translate (const CellGraph &d, GenericRepository &rep);

  /**
   *  @brief Create a new cell and return a reference to it
   *
   *  So far, cells cannot be deleted any more once they have been
   *  created.
   */
  cell_type &new_cell ();

  /**
   *  @brief Return the number of cells
   *
   *  @return The number of cells (the maximum cell index)
   */
  cell_index_type cells () const
  {
    return m_cell_ptrs.size ();
  }
  
  /**
   *  @brief Address a cell by index
   *
   *  @param i The cell index
   *  @return A reference to the cell
   */
  cell_type &cell (cell_index_type i)
  {
    return *(m_cell_ptrs [i]);
  }

  /**
   *  @brief Address a cell by index (const version)
   *
   *  @param i The cell index
   *  @return A reference to the cell
   */
  const cell_type &cell (cell_index_type i) const
  {
    return *(m_cell_ptrs [i]);
  }

  /**
   *  @brief Reserve space for the given number of cells
   */
  void reserve (size_t n)
  {
    m_cell_ptrs.reserve (n);
  }

  /**
   *  @brief Update the parent/child relationships between the cells
   *
   *  Basically this method needs to be called if changes have been
   *  made to the graph. It will update the parent instance lists
   *  from the child instance lists.
   */
  void update_relations ();

  /**
   *  @brief begin iterator of the unsorted cell list
   */
  iterator begin () 
  {
    return m_cells.begin ();
  }
  
  /**
   *  @brief end iterator of the unsorted cell list
   */
  iterator end () 
  {
    return m_cells.end ();
  }
  
  /**
   *  @brief begin iterator of the unsorted const cell list
   */
  const_iterator begin () const
  {
    return m_cells.begin ();
  }
  
  /**
   *  @brief end iterator of the unsorted const cell list
   */
  const_iterator end () const
  {
    return m_cells.end ();
  }

  /**
   *  @brief begin iterator of the bottom-up sorted cell list
   *
   *  In bottom-up traversal a cell is not delivered before
   *  the last child cell of this cell has been delivered.
   *  The bottom-up iterator does not deliver cells but cell
   *  indices actually.
   */
  bottom_up_iterator begin_bottom_up () 
  {
    update ();
    return m_top_down_list.rbegin ();
  }
  
  /**
   *  @brief end iterator of the bottom-up sorted cell list
   */
  bottom_up_iterator end_bottom_up () 
  {
    update ();
    return m_top_down_list.rend ();
  }
  
  /**
   *  @brief begin iterator of the const bottom-up sorted cell list
   *
   *  See the non-const version for a detailed description.
   */
  bottom_up_const_iterator begin_bottom_up () const
  {
    update ();
    return m_top_down_list.rbegin ();
  }
  
  /**
   *  @brief end iterator of the const bottom-up sorted cell list
   */
  bottom_up_const_iterator end_bottom_up () const
  {
    update ();
    return m_top_down_list.rend ();
  }
  
  /**
   *  @brief begin iterator of the top-down sorted cell list
   *
   *  The top-down cell list has the property of delivering all
   *  cells before they are instantiated. In addition the first
   *  cells are all top cells. There is at least one top cell.
   *  The top-down iterator does not deliver cells but cell
   *  indices actually.
   */
  top_down_iterator begin_top_down () 
  {
    update ();
    return m_top_down_list.begin ();
  }
  
  /**
   *  @brief end iterator of the top-down sorted cell list
   */
  top_down_iterator end_top_down () 
  {
    update ();
    return m_top_down_list.end ();
  }
  
  /**
   *  @brief begin iterator of the const top-down sorted cell list
   *
   *  See the non-const version for a detailed description.
   */
  top_down_const_iterator begin_top_down () const
  {
    update ();
    return m_top_down_list.begin ();
  }
  
  /**
   *  @brief end iterator of the const top-down sorted cell list
   */
  top_down_const_iterator end_top_down () const
  {
    update ();
    return m_top_down_list.end ();
  }
  
  /**
   *  @brief end iterator of the top cells
   * 
   *  The begin iterator is identical to begin_top_down()
   */
  top_down_const_iterator end_top_cells ();
  
  /**
   *  @brief end iterator of the top cells
   * 
   *  The begin iterator is identical to begin_top_down()
   */
  top_down_const_iterator end_top_cells () const;
  
  /**
   *  @brief A pseudo-const variant of update()
   *
   *  Since update virtually leaves the state of the graph
   *  unchanged we can use the const_cast hack to create 
   *  a const update() method. 
   */
  void update () const;

  /**
   *  @brief Establish the graph's internals according to the dirty flags
   *
   *  Calling update investigates the dirty flags and issues update
   *  calls on the cells and it's components as far as necessary.
   *  This will guarantee mainly that region queries can be performed
   *  on all levels of the graph.
   */
  void update ();

  /**
   *  @brief The listener for hierarchy changes
   * 
   *  This method is called by the cells if something on the
   *  hierarchy has been changed - i.e. cells have been inserted
   *  or cell instances have been inserted.
   */
  void invalidate_graph () 
  {
    m_hier_dirty = true;
  }

  /**
   *  @brief The listener for bbox changes
   *
   *  This method is called by the cell or shape containers if 
   *  something on the bbox of a cell has changed for all layers - 
   *  i.e. by inserting child instances.
   */
  void invalidate_bboxes ()
  {
    m_bboxes_dirty = true;
  }
  
  /**  
   *  @brief Delete a layer
   *
   *  This does free the shapes of the cells and remembers the
   *  layer's index for recycling.
   */
  void delete_layer (unsigned int n);

  /**
   *  @brief Tell, if a layer index is a valid index
   *
   *  @return true, if this is so
   */
  bool is_valid_layer (unsigned int n) const
  {
    return (n < layers () && m_layer_states [n] == Normal);
  }

  /**
   *  @brief Tell, if a layer index is a special layer index
   *
   *  @return true, if this is so
   */
  bool is_special_layer (unsigned int n) const
  {
    return (n < layers () && m_layer_states [n] == Special);
  }

  /**  
   *  @brief Insert a new layer
   *
   *  This creates a new index number, either from the free list
   *  of by creating a new one.
   */
  unsigned int insert_layer (bool special = false);

  /**  
   *  @brief Insert a new layer at the given index
   *
   *  If the index is unused, create a new layer there.
   */
  void insert_layer (unsigned int index, bool special = false);

  /**
   *  @brief Query the number of layers defined so far
   *  
   *  TODO: the list of 0 to nlayers-1 also contains the free layers -
   *  we should get a vector containing the layers that are acually
   *  allocated.
   */
  unsigned int layers () const
  {
    return m_layer_states.size ();
  }

  /**
   *  @brief Reserve space for n layers
   */
  void reserve_layers (unsigned int n);

  /**
   *  @brief Enable or disable per-layer cell instance box trees
   *
   *  Setting this mode to true enables per-layer cell instance
   *  box trees. This speeds up some operations but requires some
   *  memory and triggers the update of the cell instance boxes
   *  which takes some time.
   */
  void per_layer_cell_inst_trees (bool f);

  /**
   *  @brief Query the state of the per-layer cell instance trees
   */
  bool per_layer_cell_inst_trees () const
  {
    return m_per_layer_inst_box_trees;
  }

  /**
   *  @brief Collect memory usage statistics
   */
  void
  collect_mem_stat (db::MemStatistics &m) const;

private:
  enum LayerState { Normal, Free, Special };

  cell_list m_cells;
  size_t m_cells_size;
  cell_ptr_vector m_cell_ptrs;
  bool m_hier_dirty;
  bool m_bboxes_dirty;
  bool m_recursion_flag;
  cell_index_vector m_top_down_list;
  std::vector<unsigned int> m_free_indices;
  std::vector<LayerState> m_layer_states;
  bool m_per_layer_inst_box_trees;

  /**
   *  @brief Sort the cells topologically
   *
   *  Establish a sorting order top-down. The sorted 
   *  cell list can be retrieved using the begin_bottom_up 
   *  ..end_bottom_up iterator pair. Alternatively the
   *  begin_top_down..end_top_down iterator pair can be used
   *  to retrieve the cell indices in top-down order after
   *  the cells have been sorted bottom-up.
   *  This method must be called whenever the graph has changed.
   *  Before this method can be called, update_relations must have
   *  been called.
   *
   *  @return true if the cell graph is OK, false if recursive
   */
  bool topological_sort ();

};

}  // namespace db

#endif

==> klayout-0.14.tar.gz: klayout-0.14/src/dbCellGraph.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef HDR_dbCellGraph
#define HDR_dbCellGraph

#include <dbObject.h>
#include <dbCell.h>
#include <dbLayoutStateModel.h>
#include <tlException.h>
#include <tlVector.h>

#include <list>
#include <vector>

namespace db
{

class MemStatistics;
template <class Coord> class generic_repository;
typedef generic_repository<db::Coord> GenericRepository;

/**
 *  @brief A cell graph class
 *
 *  The cell graph is a container for a cell graph.
 *  The cell graph is constructed by creating cells
 *  and adding child instances to it.
 */

class CellGraph
  : public db::Object,
    public db::LayoutStateModel
{
public:
  typedef db::Box box_type;
  typedef cell_inst<CellGraph> cell_inst_type;
  typedef cell<CellGraph> cell_type;
  typedef std::list<cell_type> cell_list;
  typedef cell_list::iterator iterator;
  typedef cell_list::const_iterator const_iterator;
  typedef unsigned int cell_index_type;
  typedef tl::vector<cell_index_type> cell_index_vector;
  typedef cell_index_vector::reverse_iterator bottom_up_iterator;
  typedef cell_index_vector::const_reverse_iterator bottom_up_const_iterator;
  typedef cell_index_vector::iterator top_down_iterator;
  typedef cell_index_vector::const_iterator top_down_const_iterator;
  typedef tl::vector<cell_type *> cell_ptr_vector;

  /**
   *  @brief Default ctor: create an empty graph
   */
  CellGraph (db::Manager *manager = 0);

  /**
   *  @brief Copy ctor
   */
  CellGraph (const CellGraph &d);

  /**
   *  @brief Assignment operator
   */
  CellGraph &operator= (const CellGraph &d);

  /**
   *  @brief Translation operator
   */
  void translate (const CellGraph &d, GenericRepository &rep);

  /**
   *  @brief Create a new cell and return a reference to it
   *
   *  So far, cells cannot be deleted any more once they have been
   *  created.
   */
  cell_type &new_cell ();

  /**
   *  @brief Return the number of cells
   *
   *  @return The number of cells (the maximum cell index)
   */
  cell_index_type cells () const
  {
    return m_cell_ptrs.size ();
  }
  
  /**
   *  @brief Address a cell by index
   *
   *  @param i The cell index
   *  @return A reference to the cell
   */
  cell_type &cell (cell_index_type i)
  {
    return *(m_cell_ptrs [i]);
  }

  /**
   *  @brief Address a cell by index (const version)
   *
   *  @param i The cell index
   *  @return A reference to the cell
   */
  const cell_type &cell (cell_index_type i) const
  {
    return *(m_cell_ptrs [i]);
  }

  /**
   *  @brief Reserve space for the given number of cells
   */
  void reserve (size_t n)
  {
    m_cell_ptrs.reserve (n);
  }

  /**
   *  @brief Update the parent/child relationships between the cells
   *
   *  Basically this method needs to be called if changes have been
   *  made to the graph. It will update the parent instance lists
   *  from the child instance lists.
   */
  void update_relations ();

  /**
   *  @brief begin iterator of the unsorted cell list
   */
  iterator begin () 
  {
    return m_cells.begin ();
  }
  
  /**
   *  @brief end iterator of the unsorted cell list
   */
  iterator end () 
  {
    return m_cells.end ();
  }
  
  /**
   *  @brief begin iterator of the unsorted const cell list
   */
  const_iterator begin () const
  {
    return m_cells.begin ();
  }
  
  /**
   *  @brief end iterator of the unsorted const cell list
   */
  const_iterator end () const
  {
    return m_cells.end ();
  }

  /**
   *  @brief begin iterator of the bottom-up sorted cell list
   *
   *  In bottom-up traversal a cell is not delivered before
   *  the last child cell of this cell has been delivered.
   *  The bottom-up iterator does not deliver cells but cell
   *  indices actually.
   */
  bottom_up_iterator begin_bottom_up () 
  {
    update ();
    return m_top_down_list.rbegin ();
  }
  
  /**
   *  @brief end iterator of the bottom-up sorted cell list
   */
  bottom_up_iterator end_bottom_up () 
  {
    update ();
    return m_top_down_list.rend ();
  }
  
  /**
   *  @brief begin iterator of the const bottom-up sorted cell list
   *
   *  See the non-const version for a detailed description.
   */
  bottom_up_const_iterator begin_bottom_up () const
  {
    update ();
    return m_top_down_list.rbegin ();
  }
  
  /**
   *  @brief end iterator of the const bottom-up sorted cell list
   */
  bottom_up_const_iterator end_bottom_up () const
  {
    update ();
    return m_top_down_list.rend ();
  }
  
  /**
   *  @brief begin iterator of the top-down sorted cell list
   *
   *  The top-down cell list has the property of delivering all
   *  cells before they are instantiated. In addition the first
   *  cells are all top cells. There is at least one top cell.
   *  The top-down iterator does not deliver cells but cell
   *  indices actually.
   */
  top_down_iterator begin_top_down () 
  {
    update ();
    return m_top_down_list.begin ();
  }
  
  /**
   *  @brief end iterator of the top-down sorted cell list
   */
  top_down_iterator end_top_down () 
  {
    update ();
    return m_top_down_list.end ();
  }
  
  /**
   *  @brief begin iterator of the const top-down sorted cell list
   *
   *  See the non-const version for a detailed description.
   */
  top_down_const_iterator begin_top_down () const
  {
    update ();
    return m_top_down_list.begin ();
  }
  
  /**
   *  @brief end iterator of the const top-down sorted cell list
   */
  top_down_const_iterator end_top_down () const
  {
    update ();
    return m_top_down_list.end ();
  }
  
  /**
   *  @brief end iterator of the top cells
   * 
   *  The begin iterator is identical to begin_top_down()
   */
  top_down_iterator end_top_cells ();
  
  /**
   *  @brief end iterator of the top cells
   * 
   *  The begin iterator is identical to begin_top_down()
   */
  top_down_const_iterator end_top_cells () const;
  
  /**
   *  @brief Provide a const version of the update method
   *
   *  This pseudo-const version is required in order to automatically call
   *  update within the begin method for example.
   */
  void update () const;

  /**
   *  @brief Establish the graph's internals according to the dirty flags
   *
   *  This reimplements the LayoutStateModel interface.
   *  Calling do_update investigates the dirty flags and issues update
   *  calls on the cells and it's components as far as necessary.
   *  This will guarantee mainly that region queries can be performed
   *  on all levels of the graph.
   */
  virtual void do_update ();

  /**  
   *  @brief Delete a layer
   *
   *  This does free the shapes of the cells and remembers the
   *  layer's index for recycling.
   */
  void delete_layer (unsigned int n);

  /**
   *  @brief Tell, if a layer index is a valid index
   *
   *  @return true, if this is so
   */
  bool is_valid_layer (unsigned int n) const
  {
    return (n < layers () && m_layer_states [n] == Normal);
  }

  /**
   *  @brief Tell, if a layer index is a special layer index
   *
   *  @return true, if this is so
   */
  bool is_special_layer (unsigned int n) const
  {
    return (n < layers () && m_layer_states [n] == Special);
  }

  /**  
   *  @brief Insert a new layer
   *
   *  This creates a new index number, either from the free list
   *  of by creating a new one.
   */
  unsigned int insert_layer (bool special = false);

  /**  
   *  @brief Insert a new layer at the given index
   *
   *  If the index is unused, create a new layer there.
   */
  void insert_layer (unsigned int index, bool special = false);

  /**
   *  @brief Query the number of layers defined so far
   *  
   *  TODO: the list of 0 to nlayers-1 also contains the free layers -
   *  we should get a vector containing the layers that are acually
   *  allocated.
   */
  unsigned int layers () const
  {
    return m_layer_states.size ();
  }

  /**
   *  @brief Reserve space for n layers
   */
  void reserve_layers (unsigned int n);

  /**
   *  @brief Enable or disable per-layer cell instance box trees
   *
   *  Setting this mode to true enables per-layer cell instance
   *  box trees. This speeds up some operations but requires some
   *  memory and triggers the update of the cell instance boxes
   *  which takes some time.
   */
  void per_layer_cell_inst_trees (bool f);

  /**
   *  @brief Query the state of the per-layer cell instance trees
   */
  bool per_layer_cell_inst_trees () const
  {
    return m_per_layer_inst_box_trees;
  }

  /**
   *  @brief Signal the start of an operation bringing the layout into invalid state
   *
   *  This method should be called whenever the layout is
   *  about to be brought into an invalid state. After calling
   *  this method, "under_construction" returns false which 
   *  tells foreign code (such as update which might be called
   *  asynchroneously for example because of a repaint event)
   *  not to use this layout object.
   *  This state is cancelled by the end_changes () method.
   *  The start_changes () method can be called multiple times
   *  and must be cancelled the same number of times.
   *  If a transaction is used to bracket the start and end of
   *  an operation, the start/end_changes method pair does not
   *  need to be called.
   */
  void start_changes ()
  {
    ++m_invalid;
  }

  /**
   *  @brief Cancel the "in changes" state (see "start_changes")
   */
  void end_changes ()
  {
    if (m_invalid > 0) {
      --m_invalid;
    }
  }

  /**
   *  @brief Tell if the layout object is under construction
   *
   *  A layout object is either under construction if a transaction
   *  is ongoing or the layout is brought into invalid state by
   *  "start_changes".
   */
  bool under_construction () const
  {
    return m_invalid > 0 || (manager () && manager ()->transacting ());
  }

  /**
   *  @brief Collect memory usage statistics
   */
  void
  collect_mem_stat (db::MemStatistics &m) const;

private:
  enum LayerState { Normal, Free, Special };

  cell_list m_cells;
  size_t m_cells_size;
  cell_ptr_vector m_cell_ptrs;
  unsigned int m_invalid;
  cell_index_vector m_top_down_list;
  std::vector<unsigned int> m_free_indices;
  std::vector<LayerState> m_layer_states;
  bool m_per_layer_inst_box_trees;

  /**
   *  @brief Sort the cells topologically
   *
   *  Establish a sorting order top-down. The sorted 
   *  cell list can be retrieved using the begin_bottom_up 
   *  ..end_bottom_up iterator pair. Alternatively the
   *  begin_top_down..end_top_down iterator pair can be used
   *  to retrieve the cell indices in top-down order after
   *  the cells have been sorted bottom-up.
   *  This method must be called whenever the graph has changed.
   *  Before this method can be called, update_relations must have
   *  been called.
   *
   *  @return true if the cell graph is OK, false if recursive
   */
  bool topological_sort ();

};

inline void 
CellGraph::update () const
{
  if (! under_construction () && (hier_dirty () || bboxes_dirty ())) {
    db::LayoutStateModel *state_model = const_cast<db::LayoutStateModel *> ((const db::LayoutStateModel *) this);
    state_model->update ();
  }
}

}  // namespace db

#endif
